import { getChromeStorage } from '@/uilts/chromeExtension.js'
import { payAchNoticeUrl } from '@/http/configAPI'
import Report from "@/log-center/log"

export const closeAchPayNoticeHandler = async () => {
    try{
        let res = await getCurrentTab();
        if(res.url.startsWith(payAchNoticeUrl)) {
            chrome.tabs.remove(
                res.id, () => {
                    getChromeStorage('achPayInfo', (result) => {
                        let {tab} = result || {};
                        chrome.storage.local.remove("achPayInfo");
                        if(tab) {
                            chrome.tabs.highlight({ windowId: tab.windowId, tabs: tab.index })
                        }
                    })
                }
            )
        }
    } catch (error) {
        Report.reportLog({
            objectType: Report.objectType.background_function_catch,
            funcName: 'closeAchPayNoticeHandler',
            errMsg: error.message
        })
    }
}

const getCurrentTab = async () => {
    try{
        let queryOptions = { active: true, lastFocusedWindow: true };
        let [tab] = await chrome.tabs.query(queryOptions);
        return tab || {};
    } catch (error) {
        Report.reportLog({
            objectType: Report.objectType.background_function_catch,
            funcName: 'getCurrentTab',
            errMsg: error.message
        })
    }
}
